"use strict";
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
Object.defineProperty(exports, "__esModule", { value: true });
var QueryIdentifier_1 = require("./QueryIdentifier");
var QueryParam_1 = require("./QueryParam");
var CompiledQuery_1 = require("./CompiledQuery");
var SqlQuery = /** @class */ (function () {
    function SqlQuery(parts) {
        this.parts = parts;
    }
    SqlQuery.createFromTemplateString = function (strings) {
        var params = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            params[_i - 1] = arguments[_i];
        }
        var parts = [strings[0]];
        for (var i = 1; i < strings.length; i++) {
            if (params[i - 1] instanceof SqlQuery
                || params[i - 1] instanceof QueryParam_1.default
                || params[i - 1] instanceof QueryIdentifier_1.default) {
                parts.push(params[i - 1]);
            }
            else {
                parts.push(new QueryParam_1.default(params[i - 1]));
            }
            parts.push(strings[i]);
        }
        return new SqlQuery(parts);
    };
    ;
    /**
     * The delimiter is an SqlQuery rather than a string
     * in order to avoid potential SQL injections.
     */
    SqlQuery.join = function (queries, delimiter) {
        if (delimiter === void 0) { delimiter = SqlQuery.createFromTemplateString(templateObject_1 || (templateObject_1 = __makeTemplateObject([", "], [", "]))); }
        var joinedParts = [];
        for (var i = 0; i < queries.length; i++) {
            if (i > 0) {
                joinedParts.push(delimiter);
            }
            joinedParts.push(queries[i]);
        }
        return new SqlQuery(joinedParts);
    };
    ;
    ;
    SqlQuery.prototype.compile = function (indexToPlaceholder, formatIdentifier) {
        var sql = '';
        var params = [];
        var recursivelyAddParts = function (parts) {
            for (var i = 0; i < parts.length; i++) {
                if (parts[i] instanceof SqlQuery) {
                    recursivelyAddParts(parts[i].parts);
                }
                else if (parts[i] instanceof QueryParam_1.default) {
                    sql += indexToPlaceholder(params.length);
                    params.push(parts[i].param);
                }
                else if (parts[i] instanceof QueryIdentifier_1.default) {
                    sql += formatIdentifier(parts[i].identifier);
                }
                else {
                    sql += parts[i];
                }
            }
        };
        recursivelyAddParts(this.parts);
        return new CompiledQuery_1.default(sql, params);
    };
    return SqlQuery;
}());
exports.default = SqlQuery;
var templateObject_1;
